// Copyright (c) 2016-2017 <daggerbot@gmail.com>
// This software is available under the terms of the zlib license.
// See COPYING.md for more information.

use std::ptr;

use kernel32;
use winapi;

use error::{Error, Result};

/// `HMODULE`/`HINSTANCE` wrapper.
pub struct Module {
    handle: winapi::HMODULE,
    owned: bool,
}

impl Module {
    pub fn current_exe () -> Result<Module> {
        let handle;

        unsafe {
            handle = kernel32::GetModuleHandleW(ptr::null());
        }

        if handle.is_null() {
            return Err(Error::get().unwrap());
        }

        Ok(Module {
            handle: handle,
            owned: false,
        })
    }

    pub fn handle (&self) -> winapi::HMODULE { self.handle }
}

impl Drop for Module {
    fn drop (&mut self) {
        if self.owned {
            unsafe {
                if kernel32::FreeLibrary(self.handle) == 0 {
                    error!("FreeLibrary failed: {}", Error::get().unwrap());
                }
            }
        }
    }
}
